import { world, system, MinecraftEffectTypes, Entity } from "@minecraft/server";
import util from "./utilities";

const players = new Set();
const newHealths = new Map();

system.run(async function tick() {
  system.run(tick);

  for (const player of world.getPlayers()) {
    const weapon = util.getMainhand(player);
    const name = player.nameTag;
    const health = newHealths.get(name);

    if (!weapon) continue;
    if (!(weapon.typeId.includes("sword") && player.isSneaking)) {
      players.delete(name);
      continue;
    } else if (!players.has(name)) players.add(name);
    player.addEffect(MinecraftEffectTypes.weakness, 20, 5, false);
    await player.runCommandAsync("playanimation @s animation.player.weapon_blocking");

    if (!health || (system.currentTick % 2.5) != 0) continue; 
    player.getComponent("health").setCurrent(health);
    newHealths.delete(name);
  }
});

world.events.entityHurt.subscribe((event) => {
  const { damageSource, hurtEntity: player, damage } = event;
  const { damagingEntity: entity, damagingProjectile: projectile } = damageSource;
  const { nameTag: name, typeId: id } = player;
  
  if (id != "minecraft:player") return;
  if (!players.has(name)) return;
  const { current } = player.getComponent("health");
  const damager = projectile ?? entity;
  const isProyectile = !!projectile;
  const reduction = util.calculateReduction(damage, damager, isProyectile);

  player.playSound("item.shield.block");
  util.damageMainhand(player, 2);
  newHealths.set(name, current + reduction);
});

system.events.beforeWatchdogTerminate.subscribe((event) => {
  event.cancel = true;
});
